import pygame
from .settings import *
import pkg_resources

class ObjectRenderer:
    def __init__(self, game):
        self.game = game
        self.screen = game.screen
        self.wall_textures = self.load_wall_textures()
        self.sky_image = self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/sky.png'), (WIDTH, HALF_HEIGHT))
        self.sky_offset = 0

    def draw(self):
        self.draw_background()
        self.render_game_objects()

    def draw_background(self):
        self.sky_offset = (self.sky_offset + 4.0 * self.game.player.rel) % WIDTH
        self.screen.blit(self.sky_image, (-self.sky_offset, 0))
        self.screen.blit(self.sky_image, (-self.sky_offset + WIDTH, 0))
        # floor
        pygame.draw.rect(self.screen, FLOOR_COLOR, (0, HALF_HEIGHT, WIDTH, HEIGHT))

    def render_game_objects(self):
        list_objects = sorted(self.game.raycasting.objects_to_render, key = lambda t: t[0], reverse = True)
        for depth, image, pos in list_objects:
            self.screen.blit(image, pos)

    @staticmethod
    def get_texture(path, res=(TEXTURE_SIZE, TEXTURE_SIZE)):
        # loads texture from specified path and returns a scaled image
        texture = pygame.image.load(path).convert_alpha()
        return pygame.transform.scale(texture, res)
    
    def load_wall_textures(self):
        # returns a texture dictionary
        return {
            1: self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/1.png')),
            2: self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/2.png')),
            3: self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/3.png')),
            4: self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/4.png')),
            5: self.get_texture(pkg_resources.resource_filename(__name__,'resources/textures/5.png')),
        }
